// freqencoder.cpp : reads filenametextin.txt and
// generates a 2nd-order frequency data file.
// (c) 2002-2004 Shane T. Mueller, Ph.D
// stmuelle@indiana.edu


#include <stdio.h>
#include <string.h>
#include <memory.h>
#include <ctype.h>


char * FILEINNAME;
char * FILEOUTNAME;
FILE *filein;
FILE *fileout;
const int arraysize=27*27*27;
int freq[27][27][27];  // 0 indicates space (punctuation), i indicates
                       // letter of alphabet
void PrintInstructions();



int main(int argc, char* argv[])
{
  
  FILEINNAME = "textin.txt";
  FILEOUTNAME= "freq.dat";
  int h,i,j,k;
  char temp;
  int current=0;
  int oneback=0;
  int twoback=0;
  

  
  // Go through the command-line arguments and determine whether
  // there is an output filename specified. 
  for(h=1;h<argc;h++)
    {
		 if(strcmp(argv[h],"-o")==0
			 ||strcmp(argv[h],"-O")==0)
			 {
				 
				 //Make sure we have at least one more argument after the -o
				 if(argc > h )
					 {
						 FILEOUTNAME=argv[h+1];
						 
					 }
				 
			 }

		 if(strcmp(argv[h],"-i")==0
			 ||strcmp(argv[h],"-I")==0)
			 {
				 
				 //Make sure we have at least one more argument after the -i
				 if(argc > h )
					 {
						 FILEINNAME=argv[h+1];
						 
					 }
				 
			 }

		 if(strcmp(argv[h],"-h")==0
			 ||strcmp(argv[h],"-H")==0
			 ||strcmp(argv[h],"--help")==0)
			 {
				
				 PrintInstructions();

			 }

    }

  //Now, FILEOUTNAME should be the correct thing.  So, go through
  //the argv and read in each file in turn.

  //Initialize some things that shouldn't change between files
  long int n=0;
  memset(freq,0,arraysize);


  for(h=1; h<argc; h++)
    {  
      
      temp=0;
      current=0;
      oneback=0;
      twoback=0;
  
      if(strcmp(argv[h],"-o")==0
	 ||strcmp(argv[h],"-O")==0)
	{
	  h+=2;
	  if(h>=argc)
	        break;
	}
      
      //If we make it to here, then the argument should be an input filename

      FILEINNAME=argv[h];
      /* Open for read (will fail if file  does not exist) */
      if( (filein  = fopen( FILEINNAME, "r" )) == NULL )
	    printf( "The file %s was not opened\n", FILEINNAME );
	  else
	    printf( "The file %s was opened\n",FILEINNAME );
  

	  while (temp!=EOF){
    
	    
	    /*Print out some progress feedback to stderr*/


	    if(n%2000==0)
	      {
		if(n%100000==0)
		  fprintf(stderr,"\n%d ",n);

		fprintf(stderr,".");
	      }

	    
	    /*Increment counter*/
	    n++;
	
	    /*Get a character from the file stream*/
	    temp = getc(filein);
	  
	    
	    /*If the character is not the end of the file, process it*/
	    if(temp!=EOF)
	      {
		/*Change it to upper case, then recode to 0-26*/
		current   = toupper(temp);
		current   = (current<65 || current >90) ? 0: current-64 ;
		
		/*Increment the correct cell*/
		freq[twoback][oneback][current]++;
		
		/*Determine row and column of next letter*/
		twoback=oneback;
		oneback=current;
		
	      };

	   
	  }

	  /*Close the current input list*/
	  fclose(filein);
    }
  
  printf("Encoding complete.\n");
  
  /*open up the output file*/
  fileout  = fopen( FILEOUTNAME, "wb" );
  printf("Writing frequency table:\n");
  
  /*first, adjust some of the counts*/
  for(i=0;i<27;i++) freq[i][0][0]=0;   //never choose a null if you just chose one
  

  /*Also, account for the fact that any new word should begin [0][0][X]:
    make [0][0][X] equal to the average of [N][0][X] for all N  */
 
  for(i=0;i<27;i++)
    {
      n=0;
      
      for(j=0;j<27;j++)
	n+=freq[j][0][i];
	
      printf("Average: %d\n",n);
      for(j=0;j<27;j++)
	freq[j][0][i]=n;
     
    }
  

  float proportion=(float)0.0;
  
  for(i=0;i<27;i++){
    for(j=0;j<27;j++){
      
      //First, find out total number of cases here
      n=0;
      for(k=0;k<27;k++) n+=freq[i][j][k];
      
      for(k=0;k<27;k++){
	
	if(n==0){
	  proportion= ((k==0)? (float) 1.0: (float)0.0);
	}      
	else{
	  proportion=(float)freq[i][j][k]/n;
	};
	fwrite(&proportion,sizeof(float),1,fileout);

	
      };
      
 
    }
    

  }
  fclose(fileout);
  return 0;
}



void PrintInstructions()
{

  fprintf(stderr,"\n\nLetter Frequency Encoder.  Shane T. Mueller, Ph.D.\n");
  fprintf(stderr,"(C) 2002-2004 stmuelle@indiana.edu\n");
  fprintf(stderr,"\n");
  fprintf(stderr,"freq evaluates a text and creates a database with 2nd-order conditional\n");
  fprintf(stderr,"probabilities, that can be then used by wordgenerator to create word-like\n");
  fprintf(stderr,"non-words using the technique described by Shannon & Weaver, or evaluate letter\n");
  fprintf(stderr,"strings according to their likelihood, using the accompanying program 'like'\n");
  fprintf(stderr,"The program calculates second-order conditional probabilities of letters, so \n");
  fprintf(stderr,"it creates a table of the counts of a letter given its previous two letters.\n");
  fprintf(stderr,"All punctuation and unknown characters are treated as spaces\n");
  fprintf(stderr,"the matrix is adjusted so that it \n");
  fprintf(stderr,"The program will automatically use the file named 'textin.txt' for\n");
  fprintf(stderr,"its text, which might be a corpus or another interesting text like a\n");
  fprintf(stderr,"dictionary.  It will also automatically save its output to a file called freq.dat\n");
  fprintf(stderr,"These can be overridden with the -i (input) and -o (output) command-line arguments.\n");
  fprintf(stderr,"Example:  freq -i kucera-francis.txt -o kf-freq.dat\n");
  fprintf(stderr,"(note, the KF corpus is not included in this database, but the file\n");
  fprintf(stderr,"output/kf-freq.dat is generated with the present program run with that corpus.)\n");

	exit(1);
}
