/**************************************************************/
/*

    This program finds the distribution of Levenshtein (edit)
    distances between words in specified file and 
    the words in "dict.dat".  It only prints out a line for each word.

    Copyright 2003-2004 Shane T. Mueller
    stmuelle@indiana.edu  

*/
/**************************************************************/

#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>

#include "levenshtein.h"


/*Forward Declarations*/

void transformWord(char *, char *);
void printDist(int * levdist, char * word);
void  getNextDictWord(FILE * stream, char *word);

int main(int argc, char* argv[] )
{
  int CUTOFF = 2;
  /*This will go through each word in argv*/

  char  targetword[16];
  char dictword[16];
 
  unsigned int levdist[10];
  char tmpword[100];
  FILE * dictionary, *wordlist;
  int g, h,i,j,k;
  int distance;

  if( (dictionary=fopen("data/dict.txt", "rb"))==NULL)
    {
      fprintf(stderr,"File %s not found\n", "data/dict.txt");
      return 0;
    }

    
    /*Print out header row*/
  fprintf(stdout, "                ");
  for(i=0;i<10;i++)
	  {
		  fprintf(stdout, "%6d",i);
	  }
  fprintf(stdout,"\n");


  
  //Go through each command-line argument
  for(g=1;g<argc;g++)
     {
        if((wordlist = fopen(argv[g],"rb"))==NULL)
           {
              fprintf(stderr,"File %s not found\n", argv[g]);
              return 0;
           }
        
        h=0;
        while(!feof(wordlist))
           {
              h++;
              fgets(tmpword,100,wordlist); 
              
              transformWord(tmpword,targetword);
              //Now, targetword is u-cased.
              
              fprintf(stderr,"%s\n",targetword);
              

              //Set distribution to 0
              bzero(levdist, sizeof(int)*10);
              
              //Go through each word in dictionary and 
              //Increment distribution
              
              k=0;
              while(!feof(dictionary))
                 {
                    k++;
                    
                    getNextDictWord(dictionary,dictword);
                    
                    if(targetword[0]==EOF 
                       ||  k> 1000000)
                       break;
                    
                    
                    //Increment Distribution
                    distance = lev(targetword,dictword);
                    //	  fprintf(stderr,"%s  to %s: %d \n",targetword, dictword, distance);
                    levdist[distance]++;
                    
                    
                 }
              //              fprintf(stderr,"%d\n",k);
              printDist(levdist, targetword);
              rewind(dictionary);
           }
     }
  return 0;   
}
  
  
void getNextDictWord(FILE * stream, char *word)
{

  char line[100];
  int length;
  

  fgets(line,100,stream); 

  //Check to see I got something
  //  printf("XXX:%s",line);

  /*Copy up 15 characters/up to the first space into word*/
  transformWord(line, word);


}


void  transformWord(char * sourceword, char *ucaseword)
{

  /*Only use up to 10 characters*/
  int max = (10 > strlen(sourceword))? strlen(sourceword): 10;
  int i;

  for(i=0; i< max; i++)
    {
      ucaseword[i]=toupper(sourceword[i]);
      if(ucaseword[i]=='\0'
	 || ucaseword[i]==' '
	 || ucaseword[i]=='('
	 || ucaseword[i]=='\n'
	 || ucaseword[i]==13
	 || ucaseword[i]==10
	 )
	{
	  /*break out if there is a space*/
	  ucaseword[i]='\0';
	  break;
	}
    }
  //Make sure the last character is a null

  ucaseword[i]='\0';



}


void printDist(int * levdist, char * word)
{
  int i;



  fprintf(stdout, "%14s: ",word);
  for(i=0;i<10;i++)
    fprintf(stdout, "%6d ",levdist[i]);

  fprintf(stdout,"\n");

}
