/**************************************************************/
/*

    This program finds the distribution of Levenshtein (edit)
    distances between words on the command line and 
    the words in "dict.dat".  It will print out any 'neighbors', words that
    are two or less letters away.

    Copyright 2003 Shane T. Mueller
    smueller@umich.edu  

*/
/**************************************************************/

#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>

#include "levenshtein.h"


/*Forward Declarations*/

void transformWord(char *, char *);
void printDist(int * levdist, char * word);
void  getNextDictWord(FILE * stream, char *word);

int main(int argc, char* argv[] )
{
  int CUTOFF = 2;
  /*This will go through each word in argv*/

  char  targetword[16];
  char dictword[16];
 

  unsigned int levdist[16];
  FILE * dictionary;
  int h,i,j,k;
  int distance;

  if( (dictionary=fopen("data/dict.txt", "rb"))==NULL)
    {
      fprintf(stderr,"File %s not found\n", "data/dict.txt");
      return 0;
    }

    
      

  for(h=1; h<argc; h++)
    {
      
      transformWord(argv[h],targetword);
      //Now, targetword is u-cased.


      printf("%s\n",targetword);

      //Set distribution to 0
      bzero(levdist,sizeof(int)*16);

      //      for(i=0; i<16; i++)
      //   {
      //      levdist[i]=0;
      //   }


      //Go through each word in dictionary and 
      //Increment distribution
      
      k=0;
      while(!feof(dictionary))
         {
            k++;
            getNextDictWord(dictionary,dictword);
            
            if(targetword[0]==EOF 
               ||  k> 1000000)
               break;
	  

	  //Increment Distribution
	  distance = lev(targetword,dictword);
	  //	  fprintf(stderr,"%s  to %s: %d \n",targetword, dictword, distance);
	  levdist[distance]++;

	  //output anything that is a 'neighbor'
	  if(distance<=CUTOFF)
	    {
	      fprintf(stdout,"%s: %d\n",dictword,distance);
	    }
	  
	}
      fprintf(stderr,"%d\n",k);
      printDist(levdist, targetword);
      rewind(dictionary);
    }
  return 0;   
}
  
  
void getNextDictWord(FILE * stream, char *word)
{

  char line[100];
  int length;
  

  fgets(line,100,stream); 

  //Check to see I got something
  //  printf("XXX:%s",line);

  /*Copy up 15 characters/up to the first space into word*/
  transformWord(line, word);


}


void  transformWord(char * sourceword, char *ucaseword)
{

  /*Only use up to 15 characters*/
  int max = (15 > strlen(sourceword))? strlen(sourceword): 15;
  int i;

  for(i=0; i< max; i++)
    {
      ucaseword[i]=toupper(sourceword[i]);
      if(ucaseword[i]=='\0'
	 || ucaseword[i]==' '
	 || ucaseword[i]=='('
	 || ucaseword[i]=='\n'
	 || ucaseword[i]==13
	 || ucaseword[i]==10
	 )
	{
	  /*break out if there is a space*/
	  ucaseword[i]='\0';
	  break;
	}
    }
  //Make sure the last character is a null

  ucaseword[i]='\0';



}


void printDist(int * levdist, char * word)
{
  int i;


  /*Print out header row*/
  fprintf(stdout, "                 ");
  for(i=0;i<16;i++)
    fprintf(stdout, "%4d ",i);
  fprintf(stdout,"\n");

  fprintf(stdout, "%16s: ",word);
  for(i=0;i<16;i++)
    fprintf(stdout, "%4d ",levdist[i]);

  fprintf(stdout,"\n");

}
