/**************************************************************/
/*

    This program finds the matrix of Levenshtein or edit distances
    between words in a column stored in the specified file,
    and writes them to stdout

    Copyright 2001-2003 Shane T. Mueller
    smueller@umich.edu  

*/
/**************************************************************/
#define MAXLINE 100 
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>

#include "levenshtein.h"


FILE *phones_file;
FILE *levout_file;

int main(int argc, char* argv[] )
{
      char word1[10], word2[10]; 
      char **words;
      char input[150];
      int distance;
      int i,j,k;
      int length;
      int column;
      
      if((argc !=  2 ) && (argc != 3))
	{
	  fprintf(stderr, "%s takes one or two arguments:\n",argv[0]);
	  fprintf(stderr, " %s [filename] [column of words] \n", argv[0]);
	}
      else
	{

	  /*Determine which column the words are in*/
	  if(argc == 3){
	    strcpy(input,argv[2]);
	    column = atoi(input);
	  }
	  else
	    column = 1;
	  

	  /*Open up the appropriate file for input*/
	  strcpy(input,argv[1]);
	  phones_file  = fopen(input, "r");
	  
	  /*Read in the file once to determine how long it is*/
	  fprintf(stderr,"Reading Data Files\n"); 
	  length=0;
	  while(!feof(phones_file))
        {
           length++;
           fgets(input,150,phones_file);
           fprintf(stderr,"%d: %s",length,input);
           if(feof(phones_file)) break;
        }
	  /* length now tells us how many words we have*/
	  
     fprintf(stderr,"\n");
	  
     /*set up the words matrix*/
	  words = (char**)malloc(sizeof(char*)*length);
	  for(i=0;i<length;i++)
        words[i]=(char*)malloc(sizeof(char)*20);
     


     /* Read in the partial distance matrix*/
     /* the matrix file should look roughly like this:
       
       A B C D ...
     A 0 1 1 1
     B 1 0 1 1
     C 1 1 0 1
     D 1 1 1 0
     
     */

     FILE * partial_distance = fopen("data/letters.dat", "r"); 
     float * distance;
     distance = malloc(sizeof(float)*26*26);
     char line[200];
     char * charval;
     float value;
     int index = 0;
     for(i=0; i< 27; i++)
        {
           //Get the next line to parse
           fgets(line, 200,partial_distance);               
           const char * token = " ";
           charval = strtok(line,token);  //Get the first token; ignore it (just row label)
           for(j = 1; j< 27; j++)
              {

                 charval = strtok(NULL, token);
                 //printf("<%d, %d> %s \n",i,j,charval);                                 
                 
                 //ignore j==0 and i==0; they are just labels
                 if(j>0 && i > 0)
                    {
                       distance[index++] = atof(charval);
                    }
              }
        }
#if 0
     //Check to see whether we read in something reasonable
     for(i=0; i< 26*26; i++)
        {
           if(i%26==0) fprintf(stderr,"\n");
           fprintf(stderr,"%1.0f ",distance[i]);

        }
     fprintf(stderr,"\n");
#endif

	  /*Now, return to the beginning of the input file */
	  /*and scan in the words in column number column*/
	  rewind(phones_file);
	  for(i=0; i<length; i++)
        {
           for(j=1;j<column;j++)
              {
                 fscanf(phones_file, "%s",  input);  /*Dummy columns*/
              }

           fscanf(phones_file, "%s", input);   /* word          */
           strncpy(words[i],input, 20);        /* copy to array */
           fprintf(stderr,"Getting word %d of %d: %s\n",i+1,length,input);  /*Report it to the user*/
           fgets( input, 150 , phones_file );  /* rest of line  */
	      
        };
 	  
      
     fprintf(stderr,"Calculating Levenshtein Distances\n");
     
     for(i = 0; i<length; i++)
        {
           for(j = 0; j<length; j++)
              {
                 float val = lev_partial(words[i],words[j], distance);
                 fprintf(stdout, "%f ",val);
              };
           //fprintf(stderr,".");
           fflush(stderr);
           fprintf(stdout, "\n");
        };
     
     fclose(phones_file);
      
   }    
      return 0;   
}

  
