/**************************************************************/
/*

    This program finds the Levenshtein or edit distance
    between words stored in stdin, and writes them to
    stdout

    Copyright 2001-2003 Shane T. Mueller
    smueller@umich.edu  

*/
/**************************************************************/

#define MAXLINE 100 
#include <string.h>
#include <stdio.h>


/*Calculate levenshtein distance.  This works by forming a m x n matrix, 
 and working through it item by item. Basically, each cell represents the 
 partial levenshtein distance for the first i and j letters of the words.  Recursively,
 if you know the lev. distance between two words, then if you add a letter to the end of one,
 the new distance is the smallest one of three values: two different types of shifts and
 one switch. At the end, the bottom right-hand corner is the levenshtein
 distance. */
int lev(char a[],char b[]) 
{
    
   int arr[MAXLINE][MAXLINE]; 
   int i,j,l,m,n,add; 
   
   
   for (i=0;i<=strlen(a);i++) 
      {	 
         arr[0][i]=i; 
      }
   
   for (j=0;j<=strlen(b);j++) 
      {
         arr[j][0]=j; 
      }
      
   for (j=1;j<=strlen(b);j++) 
      {
         for (i=1;i<=strlen(a);i++) 
            {
               if (a[i-1] == b[j-1]) 
                  {
                     add=0; 
                  }
               else 
                  {
                     add=1; 
                  }   
               m = 1+arr[j-1][i]; 
               l = 1+arr[j][i-1]; 
               n = add+arr[j-1][i-1]; 
               arr[j][i] = (m < l ? (m < n ? m : n): (l < n ? l : n)); 
            }
      }
   
   
   
   return arr[strlen(b)][strlen(a)]; 
   
}

/*The following function calculates a 'weighted' levenshtein 
 distance, which is generally less than or equal to the actual 
 levenshtein distance. 
 
 Removing/adding a character 'costs' one point;  changing 
 from one letter to another costs a value found in the passed-in 
 matrix, which should all be less than 1 (with 0 on the diagonal)

*/
float lev_partial(char a[], char b[], const float * distance)
{
   //set aside a 2-d matrix to store all the distances
   float arr[MAXLINE][MAXLINE]; 
   int i,j;
   float l,m,n;
   float add; 
   
   //Fill in the 0th row with 0 to n: this is as bad as it can get
   for (i=0;i<=strlen(a);i++) 
      {	 
         arr[0][i]=(float)i; 
         a[i]=toupper(a[i]);
      }
   
   //Fill in the 0th column with 0 to n; this is as bad as it can get
   for (j=0;j<=strlen(b);j++) 
      {
         arr[j][0]=(float)j; 
         b[j]=toupper(b[j]);
      }
      
   //   printf("Comparing [%s] [%s]",a,b);
   //Go through each letter of b
   for (j=1;j<=strlen(b);j++) 
      {
         //For each letter of b, go through each letter of a
         for (i=1;i<=strlen(a);i++) 
            {
               int row = a[i-1]-'A';
               int col = b[j-1]-'A';
               if(row<0 || row> 25) 
                  add=1;
               else if(col<0 || col> 25) 
                  add=1;
               else
                  add = distance[row * 26 + col];
               
 
               //               printf("Add: %d\n",add);
               m = 1+arr[j-1][i];      //m measures a shift
               l = 1+arr[j][i-1];      //l is a different shift
               n = add+arr[j-1][i-1];  //n is a replacement
               //Choose the smallest one
               arr[j][i] = (m < l ? (m < n ? m : n): (l < n ? l : n)); 
            }
      }
      

#if 0
   /*This just prints out a bunch of stuff for checking*/
   printf("\n---------------------------: %d  %d\n", strlen(a), strlen(b));
   for(i=0;i<strlen(a);i++)
      {
         for(j=0;j<strlen(b); j++)
            {
               printf("%2.0f ",arr[j][i]);
            }
         printf("\n");
      }
   printf("---------------------------\n");
   printf("returning: %f\n", arr[strlen(b)-1][strlen(a)-1]);
#endif
   return arr[strlen(b)-1][strlen(a)-1]; 
} 
