// likelihoodfinder.cpp: evaluates the 2nd-order likelihood of letter strings
// (c) 2002-2004 Shane T. Mueller, Ph.D
// stmuelle@indiana.edu


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <ctype.h>
#include <math.h>

#include "loadfrequency.h"


float freq[27][27][27];  // 0 indicates space (punctuation), i indicates letter
void PrintInstructions();

int main(int argc, char* argv[])
{
  char * inputwords;
  FILE *filein;
  char * filename="data/freq.dat";
  int h,i;
  int wordlength;
  int current,oneback,twoback;
  double loglikelihood;

	
  if (!loadfrequencydata(filename))
    return 1;
  
  
  if(argc==1) PrintInstructions();
  


  //now, read in words
  for(h=1;h<argc;h++)
	  {
		  //Take filenames from argv
		  inputwords = argv[h];
		  
		  if( (filein=fopen(inputwords,"rt")) == NULL)
			  printf( "The file %s was not opened\n",inputwords );
		  else
			  {
				  char tempword[50];
				  
				  while(1){   //repeat this loop for each word in file
					  if(fgets(tempword,50,filein)==NULL)
						  break;
					  else
						  {
							  wordlength=strlen(tempword); 
							  //make sure that each word ends with 
							  //a blank space (32), and then a null (0), rather than cr
							  // or CR-LF.
							  //10 is LF, 13 is CR.  Make sure you handle 
							  //either CR/LF (dos) or JUST LF (10)
							  if(tempword[wordlength-1]==10)
								  {
									  if(tempword[wordlength-2]==13)
										  {
											  //CR-LF
											  tempword[wordlength-2]=32;
											  tempword[wordlength-1]=0;
											  tempword[wordlength]=0;
										  }
									  else
										  {
											  //Just LF
											  tempword[wordlength-1]=32;
											  tempword[wordlength]=0;
											  
										  }
								  }
							  else
								  {
									  tempword[wordlength]=32;
									  tempword[wordlength+1]=0;
								  };
							  
							  
							  wordlength=strlen(tempword)-1;               //this is the actual word length
							  
							  
							  oneback=0;
							  twoback=0;
							  loglikelihood=(double)1.0;
							  
							  //loop through each letter of the word (including a space at the end)
							  for(i=0;i<wordlength+1;i++)
								  {
									  current=toupper(tempword[i])-64;
									  if(current<0 || current>26) 
										  current=0;                     //make sure its in the alphabet or 0
									  
									  //impossible letters get a log-likelihood penalty of -1000
									  //other letters get their log-likelihood added into the total
									  if(freq[twoback][oneback][current]==0) 
										  {loglikelihood-=1000.0;} 
									  else{
										  loglikelihood+= log((double)freq[twoback][oneback][current]);
									  };
									  
									  
									  //printf("%d  %d  %d  %f %f \n",twoback,oneback,current,freq[twoback][oneback][current],loglikelihood);
									  twoback=oneback;
									  oneback=current;
									  
								  };
							  printf("%-20s %1.9f\n",tempword,exp(loglikelihood/(wordlength+1)));
							  
						  }
					  
				  }
			  }
	  }
  return 0;
}



void PrintInstructions()
{

  fprintf(stderr,"\n\nLikelihood Evaluator.  Shane T. Mueller, Ph.D.\n");
  fprintf(stderr,"(C) 2002-2004 stmuelle@indiana.edu\n");
  fprintf(stderr,"\n");
  fprintf(stderr,"like determines the 2nd-order likelihood of a letter string, according to probabilities\n");
  fprintf(stderr,"stored in data/like.dat, created by bin/freq based on a text corpus.\n");
  fprintf(stderr,"The program calculates second-order conditional probabilities of letters, so \n");
  fprintf(stderr,"it uses a table of the probabilities of a letter given its previous two letters in the.\n");
  fprintf(stderr,"word.  The program will evaluates the first word on each line of each file specified\n");
  fprintf(stderr,"on the command-line, and prints the results to stdout.\n");
  fprintf(stderr,"Usage:\n");
  fprintf(stderr,"like wordfile.txt > wordlike.txt\n");

	exit(1);
}
