// 
//  WordGenerator
//  Ported to GCC from VC++ original 
//  (c) 2001-2004 Shane T. Mueller, Ph.D.
//


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <time.h>

#include "loadfrequency.h"
#include "loaddictionary.h"


void PrintHeader();
int loaddictionary(char * dictfile, long int* byteposition);
int checkword(char * pWord, FILE * stream, long int* byteposition);


float freq[27][27][27];  // 0 indicates space (punctuation), i indicates




int main(int argc, char* argv[])
{

  /*Initial Declarations*/
  int current, oneback, twoback;
  float myRandom;
  float cumulative;
  
  
  int i,j,k,l;  /*miscellaneous counters*/

  FILE *dictionary;
  
  char word[50];


  //Default values for  MAXLENGTH, MINLENGTH, and NUMBEROFWORDS
  int MINLENGTH = 5;
  int MAXLENGTH = 9;
  int NUMBEROFWORDS = 20;
  char * FILENAME="data/freq.dat";
  char * DICTFILENAME="data/dict.txt";
  long int byteposition[27];

  //Print out standard header
  if(argc>1)
    if(strcmp(argv[1],"-h")==0
       || strcmp(argv[1],"-H")==0
       || strcmp(argv[1],"--help")==0
       )
      PrintHeader();

  for( i = 2; i < argc; i+=2)
    {

      //Get argument for -n
      if(strcmp(argv[i-1],"-n") == 0
	 || strcmp(argv[i-1],"-N")==0)
	NUMBEROFWORDS = atoi(argv[i]);

      //Get argument for -f
      else if(strcmp(argv[i-1],"-f")==0)
	FILENAME=argv[i];
	
      
      //Get argument for MINLENGTH
      else if(strcmp(argv[i-1],"-min")==0 
	 ||strcmp(argv[i-1],"-MIN")==0 
	 ||strcmp(argv[i-1],"-Min")==0 
	 ||strcmp(argv[i-1],"-from")==0 
	 ||strcmp(argv[i-1],"-FROM")==0 
	 ||strcmp(argv[i-1],"-From")==0 
	 )
	MINLENGTH= atoi(argv[i]);


      //Get argument for MAXLENGTH
      else if(strcmp(argv[i-1],"-max")==0 
	 ||strcmp(argv[i-1],"-MAX")==0 
	 ||strcmp(argv[i-1],"-Max")==0 
	 ||strcmp(argv[i-1],"-to")==0 
	 ||strcmp(argv[i-1],"-TO")==0 
	 ||strcmp(argv[i-1],"-To")==0 
	 )
	MAXLENGTH= atoi(argv[i]);
      else 
	{
	  fprintf(stderr, "Incorrect Command Line Options\n");
	  return 1;
	}
    }

  //If there are the wrong number of command-line arguments, just quit
  if(argc % 2 == 0
     || MINLENGTH > MAXLENGTH
     || MINLENGTH < 1
     || MAXLENGTH < 2)
    {
      fprintf(stderr, "Incorrect Command Line Options: erroneous lengths\n");
      return 1;
    }



  srand( (unsigned)time( NULL ) );    //seed random number generator


  /*Load the frequency data from FILENAME*/
  if(!loadfrequencydata(FILENAME))
    return 1;
  

  /*Load the dictionary byteposition array*/
  if(!loaddictionary(DICTFILENAME, byteposition))
    return 1;

  
  /*Now, open the dictionary stream*/
  if( (dictionary=fopen(DICTFILENAME, "rb"))==NULL)
    {
      fprintf(stderr,"File %s not found\n",DICTFILENAME);
      return 1;
    }  
  

 //Right now, dictionary should be readable.
  fseek(dictionary,1000,SEEK_SET);      //Set file position pointer to letter access point

  char line[100];
 

  fgets(line,100,dictionary);
  strcpy(word,"");
	
  i=0;
  current=0;
  oneback=0;
  twoback=0;
  
  
  //Now, generate a word
  
  k=0;
  while(k<NUMBEROFWORDS){
    i=0;
    
    
    //generate words that are up to two longer than the max length;
    //This assures that the words end like real words.
    while(i<MAXLENGTH+2){
      myRandom = (float)rand() / RAND_MAX;
      j=0;
      cumulative=0.0;
      
      
      //now, use random number to find next character
      for(j=0;j<27;j++)
	{


	  cumulative= cumulative + freq[twoback][oneback][j];
	  // letter of alphabet
	  
	  
	  if(cumulative>myRandom)
	    {current=j;				//if I've chosen a letter, set up for the next letter 
	    twoback=oneback;
	    oneback=current;
	    
		  
	    break;};
	  
	};

      word[i]=(char)(j+64);					//add new letter to word
      //now, check if the word is over with;
      if(word[i]==64){
	
	word[i]=0;
	break;};
      i++;
    };
    
    //now, Check if it is within the proper bounds.
    if( ( strlen(word)>=MINLENGTH) &&
	( strlen(word)<=MAXLENGTH) &&
	!checkword(word, dictionary, byteposition))
      {
	//it is a valid nonword.  display it.
	printf("%s\n",word);
	k++;
      }
    else
      {
	//printf("Word is NOT good: %s\n",word);
      };
  };	
  fprintf(stderr,"Done\n");
  close(dictionary);
  return 0;
}








//This takes a 'word' and checks whether it is in the
//dictionary.  If it is, it returns 1, otherwise it returns 0.
int checkword(char * pWord, FILE * stream, long int* byteposition){

	int i=0;
	char line[100];
	int WordL=strlen(pWord);
	int DictWordL=0;

	//First, jump to the best place in the dictionary to start looking

	i=pWord[0]-65+1;               //A (character 65) is the second key in the positions array--or element 1
	i=(i<0) ? 1: (i>26 ? 0: i);    //Find best letter to start search


	fflush(stdout);
	if(stream)
	  fseek(stream,byteposition[i],SEEK_SET);      //Set file position pointer to letter access point

	//Now, scan sequentially until either:
	//     	1.  You find a spelling match:        return 1
	//	2.  You are later in the alphabet:    return 0


	//Here, we first need to identify the length of the word
	//This should be done for both the input words and the dictionary words


	while(1)
	{
		fgets(line,100,stream);
		DictWordL=strcspn(line," ");  //Find length of Dictionary word 

///This can be uncommented for debugging
#if 0
		printf("##[%s] [%s] [%d]",word,line,strncmp(line,pWord,WordL));
#endif

		int comparison = strncmp(line, pWord, WordL);
		

		
		if(comparison<0)         //keep searching 
		  ;//noop;
		else if(comparison == 0)
		  {            //stop searching--word found 	 	
		    fprintf(stderr,"Word found in dictionary\n");
		    return 1;
		  }
		else
		  {
		    //stop searching--word passed 
		    //printf("Word surpassed in dictionary\n");
		    //change line to null character line 
		    return 0;
		  }

	};

  //If I get to this point, I couldn't find it in the dictionary
 return 0;
}


void PrintHeader()
{
  fprintf(stderr,"\n\n\nNon-word generator.  Shane T. Mueller, Ph.D.\n");
  fprintf(stderr,"(C) 2002-2004 stmuelle@indiana.edu\n");
  fprintf(stderr,"wordgenerater creates word-like nonwords,\n");
  fprintf(stderr,"and print them to stdout.  If called with no arguments,\n");
  fprintf(stderr,"20 such non-words will be produced, between the lengths\n");
  fprintf(stderr,"of 5 and 9. The number of generated non-words can be \n");
  fprintf(stderr,"controlled by using the -n command-line option, and the\n");
  fprintf(stderr,"lengths of the shortest and longest words can be controlled\n");
  fprintf(stderr,"using the -min and -max options\n");
  fprintf(stderr,"Examples:\n");
  fprintf(stderr,"[./wordgen -n 1000 -min 5 -max 5] produces 1000 5-letter non-words\n");
  fprintf(stderr,"[./wordgen -min 2 -max 15 -n 100] produces 100 non-words between\n");
  fprintf(stderr,"                                  2 and 15 letters long\n\n");
  fprintf(stderr,"This program requires two data files to be in the /data directory: a 2nd order ");
  fprintf(stderr,"conditional probability \n");
  fprintf(stderr,"matrix, stored in freq.dat, and a dictionary, stored in dict.txt. \n");
  fprintf(stderr,"The frequency file is generated by the accompanying 'freq' program \n");
  fprintf(stderr,"Both of these files are available in the data directory. which contains 2nd order");
  fprintf(stderr,"probabilities based on either the Kucera-Francis corpus, the words in the ");
  fprintf(stderr,"CMU machine-readable dictionary, or both.  Copy or symlink the a .dat file");
  fprintf(stderr,"to be named freq.dat to change dictionaries.");
}
